

div.dataTables_filter label {
  margin-top: 6px;
  margin-left: 6px;
}

div.dataTables_filter input {
  display: inline;
  margin-top: -6px;
  margin-left: 6px;
  margin-bottom: 4px;
}

div.dataTables_length select {
  display: inline;
  width: 75px;
}
 
div.dataTables_info {
  padding-top: 8px;
}
 
div.dataTables_paginate {
  float: right;
  margin: 0;
}

div.dataTables_paginate a {
  line-height: 26px;
}

th.header {
  background: url('../images/sort_both.png') no-repeat center right;
}
 
th.headerSortDown {
  background: url('../images/sort_desc.png') no-repeat center right;
}

th.headerSortUp {
  background: url('../images/sort_asc.png') no-repeat center right;
}


